#!/bin/sh

# set & check where all the good stuff is
sbox=/scratchbox/users/$USER
utils=/scratchbox/tools/bin


# Check if we are in normal or sudo mode.

if [ -f "/scratchbox/.sudo_mode" ] ; then
	# Sudo mode

	SUDO=sudo
	CHROOT_SUDO="/scratchbox/sbin/chroot-sudo-uid"
	export HOME=/home/$USER

	if [ ! -f "$CHROOT_SUDO" ]; then
		echo "ERROR: '$CHROOT_SUDO' is missing."
		exit 1
	fi

	# register arm binaries (only for sudomode)

	$SUDO /scratchbox/sbin/register_misc_runner

	# do mount --bind:ing (only for sudomode)
	mkdir -p $sbox/proc $sbox/tmp $sbox/scratchbox $sbox/scratchbox/dev

	if ! mount | grep -q " on $sbox/proc" ; then
		$SUDO mount --bind /proc $sbox/proc
	fi
	if ! mount | grep -q " on $sbox/tmp" ; then
		$SUDO mount --bind /tmp $sbox/tmp
	fi
	if ! mount | grep -q " on $sbox/scratchbox" ; then
		$SUDO mount --bind /scratchbox $sbox/scratchbox
	fi
	if [ -d /scratchbox_local/dev ]; then
		if ! mount | grep -q " on $sbox/scratchbox/dev" ; then
			$SUDO mount --bind /scratchbox_local/dev $sbox/scratchbox/dev
		fi
	fi

	CHROOT="sudo $CHROOT_SUDO"
else
	# Normal mode

	CHROOT="/scratchbox/sbin/chroot-uid"

	if [ ! -f "$CHROOT" ]; then
		echo "ERROR: '$CHROOT' is missing."
		exit 1
	fi
	if [ ! -u "$CHROOT" ]; then
		echo "ERROR: '$CHROOT' is not suid ROOT."
		exit 1
	fi
	if [ ! -x "$CHROOT" ]; then
		echo "ERROR: You don't have permission to run '$CHROOT'."
		echo "Maybe you have not logged out and in again after being added to sbox group."
		exit 1
	fi
fi

# Sanity checking

if [ `id -u` == "0" ] ; then
	echo "Not allowed to run this as root! Use some normal user!"
	exit 1
fi

if [ ! -d "$sbox" ]; then
	echo "ERROR: Scratchbox dir '$sbox' is missing."
	exit 1
fi
if [ ! -d "$sbox/$utils" ]; then
	echo "ERROR: Scratchbox dir '$sbox/$utils' is missing."
	exit 1
fi
if [ ! -c "$sbox/scratchbox/dev/zero" ] ; then
	echo "ERROR: /dev is probably missing.!"
	exit 1
fi


# go to scratchbox
cd $sbox
if [ $? != 0 ]; then
	echo "ERROR: Could not change to '$sbox' directory."
 	exit 1
fi

# check shell
if [ ! -x "scratchbox/tools/bin/sh" ]; then
	echo "ERROR: Sandbox shell '$sbox/$utils/sh' missing (Maybe sb mounts are not active?)."
	exit 1
fi


# command to run services in chroot jail
RUN="$CHROOT $PWD"

# print sbox instructions
echo "
Welcome to *Scratchbox*, the cross-compilation SDK!

Scratchbox is a self-contained mini-distribution that contains all the
tools needed to configure and cross-compile Open Source software using
GNU make and *autoconf*.  All the software configuration, compilation
and installation is done in a sandbox environment exactly like on your
target device.  This is achieved by using chroot to limit system
visibility for configuration scripts you run from this shell, into what
exists inside the Scratchbox sandbox.

The Scratchbox utilities are:
  sbox-config -- Change your compilation target

"

# Set all variables for the initial shell (redirection stuff is important)
eval $(grep ^export $sbox/sb_etc/profile)

# Copy few files from host (Only things that most definetely are there!)
# /etc/resolv.conf and /etc/hosts ...
if [ ! -f "$sbox/sb_etc/resolv.conf" ] ; then
	cp /etc/resolv.conf "$sbox/sb_etc/"
fi

if [ ! -f "$sbox/sb_etc/hosts" ] ; then
	cp /etc/hosts "$sbox/sb_etc/"
fi

if [ ! -f "$sbox/sb_etc/localtime" ] ; then
	if [ -e /etc/localtime ] ; then
		cp /etc/localtime "$sbox/sb_etc/"
	fi
fi

# Copy and create other stuff for user
if [ ! -e "$sbox/home/$USER/.terminfo" ] ; then
	ln -s /scratchbox/etc/terminfo "$sbox/home/$USER/.terminfo"
fi

if [ ! -f "$sbox/sb_etc/group" ] ; then
	echo "root:*:0:root" > "$sbox/sb_etc/group"
	echo "`groups | cut -d ' ' -f 1`:*:$GROUPS:" >> "$sbox/sb_etc/group"
fi

# Select current target CPU
if [ ! -L "$sbox/targets/links/scratchbox.config" ]; then
	export LD_PRELOAD=/scratchbox/tools/lib/libsb.so.0
	$RUN scratchbox/tools/bin/sh -l $utils/sbox-config --create-target=HOST --compiler-name=host-gcc --cputransparency-method=none --devkits=none
	$RUN scratchbox/tools/bin/sh -l $utils/sbox-config --select-target=HOST
	unset LD_PRELOAD
fi

## Warn about missing target in .sbrsh
#. $sbox/`ls -la $sbox/targets/links/scratchbox.config | cut -d / -f 8-`
#
#if [ ! -f "/scratchbox/users/$USER/home/$USER/.sbrsh" ] ; then
#	echo "WARNING: File /scratchbox/users/$USER/home/$USER/.sbrsh"
#	echo "is missing. It is needed by the cpuTransparency!"
#	echo
#else
#	SBRSH_TARGET=`cat /scratchbox/users/$USER/home/$USER/.sbrsh | grep "^$SBOX_TARGET_NAME[ \t]"`
#	if [ "x$SBRSH_TARGET" = "x" ] ; then
#		if [ "$SBOX_CPU" != "i386" ] ; then
#			echo "WARNING: You don't have this target in the"
#			echo "/scratchbox/users/$USER/home/$USER/.sbrsh file."
#			echo "This is needed by the cpuTransparency!"
#			echo
#		fi
#	fi
#fi

#echo "---[ EXPORTING TARGET DIRECTORIES ]---"
#/scratchbox/sbin/sb-exportfs
#echo "--------------------------------------"

#  sbox-image -- Create target root image

# automatic scratchbox command execution
if [ ! -z $1 ]; then
	argc="-c $*"
else
	args=""
fi

# run SDK shell
export LD_PRELOAD=/scratchbox/tools/lib/libsb.so.0
export SBRSH_ENV_LD_PRELOAD=(UNSET)

export _SBOX_RESTART_FILE=/tmp/restart-sbox.$USER.$$

trap true HUP

$RUN scratchbox/tools/bin/sh -l $args
while [ -f $_SBOX_RESTART_FILE ]; do
	rm $_SBOX_RESTART_FILE
	echo "Shell restarting..."
	$RUN scratchbox/tools/bin/sh -l
done

unset LD_PRELOAD

# kill the script process group (everything run from this script)
trap true TERM
kill -- -$$
